unit good;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ActnList, ComCtrls;

type
  TForm1 = class(TForm)
    btnMemorySafe: TButton;
    ListBox1: TListBox;
    Button1: TButton;
    ActionList1: TActionList;
    actionBlockListBox: TAction;
    Edit1: TEdit;
    Button2: TButton;
    actionNumbersOnly: TAction;
    StatusBar1: TStatusBar;
    btnGoodFile: TButton;
    Edit2: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure btnMemorySafeClick(Sender: TObject);
    procedure actionBlockListBoxUpdate(Sender: TObject);
    procedure actionBlockListBoxExecute(Sender: TObject);
    procedure actionNumbersOnlyExecute(Sender: TObject);
    procedure actionNumbersOnlyUpdate(Sender: TObject);
    procedure btnGoodFileClick(Sender: TObject);
  private
    { Private declarations }
    procedure HandleExceptions(Sender :TObject; sException :Exception);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses utils;

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  Application.OnException := HandleExceptions;
  PosForm(Self, alLeft);
end;

{the following simply logs messages that we've indicated are okay. We've done
this by only allowing intentional exceptions to get here. That is, if it's a
ECustomException, then we made it.

If it's not an ECustomException and it gets here then something went wrong and
we need to decide how to handle them. In this case, we still log the error but
then dump the application.}

procedure TForm1.HandleExceptions(Sender :TObject; sException :Exception);
begin
  LogError(sException);
{would normally terminate the application if the exception is not intentional!
In this case we simply show the exception as if no handling was in place at all.}
  if not(sException is ECustomException) then
    Application.ShowException(sException);
    //Application.Terminate;
end;


procedure TForm1.btnMemorySafeClick(Sender: TObject);
var
  mStream :TMemoryStream;
  fStream :TFileStream;
begin

mStream := nil;
fStream := nil;

    mStream := TMemoryStream.Create;
    try
      //forgot to do the fStream := TFileStream.Create(blah) !
      try
        try
          mStream.CopyFrom(fStream, 0); //thus this will cause an AV
        except
          on E :Exception do
            raise ECustomException.Create('btnMemorySafe', E.ClassName, E.Message);
          end;
      finally
        fStream.Free;
        end;
    finally
      FreeAndNil(mStream);
      end;
end;

procedure TForm1.actionBlockListBoxUpdate(Sender: TObject);
begin
  with (Sender as TAction) do begin
    if (ListBox1.ItemIndex < 0) then
      Enabled := False
    else
      Enabled := True;
  end;
end;

procedure TForm1.actionBlockListBoxExecute(Sender: TObject);
begin
  with ListBox1 do begin
    Self.Caption := Items[ItemIndex];
    Items.Delete(ItemIndex);
  end;
end;

procedure TForm1.actionNumbersOnlyExecute(Sender: TObject);
begin
  Self.Caption := Edit1.Text;
end;

procedure TForm1.actionNumbersOnlyUpdate(Sender: TObject);
begin

  with (Sender as TAction) do begin
    try
      StrToInt(Edit1.Text);
      Enabled := True;
    except
      Enabled := False;
      end;
  end;
end;

procedure TForm1.btnGoodFileClick(Sender: TObject);
var
  MyFile :TFileStream;
  fName, s :String;
begin

  if (Length(Edit2.Text) = 0) then
    fName := GetParentFolder + 'testfile.txt'
  else
    fName := Edit2.Text;

  try
    MyFile := TFileStream.Create(fName, fmOpenReadWrite);
    try
      SetLength(s, MyFile.Size);
      MyFile.Read(s[1], MyFile.Size);
      Self.Caption := s;
    finally
      FreeAndNil(MyFile);
      end;
  except
    try
      RaiseLastWin32Error;
    except
      on E :EWin32Error do begin
        case GetLastError of
          2,3,15, 55: begin //file not found, path not found, invalid drive, device does not exist
            s := 'The file was not found in the location you gave'#13#10#13#10;
            s := s + '"' + fName + '"' + #13#10#13#10;
            s := s + 'Please verify that the file exists in that location or ';
            s := s + 'please choose a file that exists.';
          end;
          4: begin //too many files already open
            s := 'You currently have too many files open on the system. Please ';
            s := s + 'close some open files and try to open the file again.';
          end;
          5,19,65 : begin //Access is denied, write protected device?, Network access denied?
            s := 'Access to the file has been denied. Please ensure that you ';
            s := s + 'have read and write privelidges to the file'#13#10#13#10;
            s := s + '"' + fName + '"' + #13#10#13#10;
            s := s + 'by right clicking on the file and checking the properties ';
            s := s + 'option.';
          end;
          53,54,67 :begin //bad network path, network busy
            s := 'The network path is currently unavailable. Please check your ';
            s := s + 'network connections and try again.';
          end;
        else
          s := 'Error Message : ' + E.Message;
        end;
        //size message
        s := WrapText(s, #13#10, ['.',' ',#9,'-'], (Screen.Width div 3));
        //display and log message
        Application.MessageBox(PChar(s),
                              'An Error Occurred trying to open the file',
                              MB_OK OR MB_ICONINFORMATION);
        raise ECustomException.Create('btnGoodFileClick', E.ClassName, E.Message);
        end;
      end;
    end;
end;//func

end.
