unit utils;

interface
  uses
    Controls,
    Forms,
    SysUtils;

  //simple helper functions
  procedure PosForm(sForm :TForm; sAlign :TAlign);
  procedure LogError(sException :Exception);
  function GetParentFolder:String;

type
  ECustomException = class(Exception)
    private
      FCallingFunction, FExceptionType, FNewMessage :String;
      FTime :TDateTime;
      function FGetTime:String;
      function SortOutWin32ErrorMessage(sMessage :String):String;
    public
      constructor Create(sFunc, sType, sNewMessage :String);
      function ErrorString :String;
      property CallingFunction  :String  read FCallingFunction;
      property ExceptionType    :String  read FExceptionType;
      property NewMessage       :String  read FNewMessage;
      property Time             :String  read FGetTime;
  end;

implementation

uses
  IniFiles;

{simple position of form to avoid using TForm.Align which causes problems to
owned controls when setting width before form is shown}
procedure PosForm(sForm :TForm; sAlign :TAlign);
begin
  with sForm do begin
    Width  := (Screen.Width div 2);
    Height := Screen.Height;
    Top    := 0;
    case sAlign of
      alLeft : Left := 0;
      alRight : Left := (Screen.Width - Width);
    end;
  end;
end;

{the following logs errors which occur. In this application it is called only
from the Application.OnException handler below. The ini file produced will be
[Function]
Time=Exception Class, Message}
procedure LogError(sException :Exception);
var
  MyIni :TIniFile;
  CurrPath :String;
begin
  CurrPath := ExtractFilePath(ParamStr(0)) + '\error.log';
  MyIni := TIniFile.Create(CurrPath);
  try
    if (sException is ECustomException) then begin
      with (sException as ECustomException) do
        MyIni.WriteString(CallingFunction, Time, ErrorString);
    end
    else begin
      with sException do begin
      MyIni.WriteString('UNKNOWN', FormatDateTime('dd-mm-yy hh:mm:ss', Now),
                        ClassName + ' : ' + Message);
      end;
    end;

  finally
    FreeAndNil(MyIni);
    end;

end;
{simple helper to return the parent folder of the current folder. eg
assuming ParamStr returns d:\borland\delphi\projects\articles\robust.exe
GetParentFolder
would both return
d:\borland\delphi\projects\}
function GetParentFolder:String;
begin
  Result := ExtractFilePath(ParamStr(0));
  Result := ExcludeTrailingBackslash(Result);
  Result := ExtractFilePath(Result);

end;


{ ECustomException
custom exception class. Used here to simplify logging of any exception that
may occur by raising it out to the Application.OnException handler.}

constructor ECustomException.Create(sFunc, sType, sNewMessage: String);
var
  RepFlags :TReplaceFlags;
begin
  inherited Create(sNewMessage);

  RepFlags := [rfReplaceAll, rfIgnoreCase];
  FCallingFunction := sFunc;
  FExceptionType := sType;
  FNewMessage := SortOutWin32ErrorMessage(sNewMessage); //ready for Win32Error messages
  FTime := Now();
end;

function ECustomException.ErrorString: String;
begin
  Result := FExceptionType + ' : ' + FNewMessage;
end;

function ECustomException.FGetTime: String;
begin
  Result := FormatDateTime('dd-mm-yy hh:mm:ss', FTime);
end;

function ECustomException.SortOutWin32ErrorMessage(sMessage :String):String;
var
  RepFlags :TReplaceFlags;
begin
  RepFlags := [rfIgnoreCase, rfReplaceAll];
  Result := AdjustLineBreaks(sMessage);
  Result := StringReplace(Result, #13#10, ' ', RepFlags);
end;


end.


